/********************************************************************
 * FileName:        main.c
 * Dependencies:    See INCLUDES section below
 * Processor:       MCP19111
 * Compiler:        XC8 v1.11
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement:
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") for its PICmicro Microcontroller is intended and
 * supplied to you, the Company's customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *********************************************************************
 *
 * Version              Date        Comment
 * 1.000                05.12.2012  Initial release
 ********************************************************************/

#include "pic.h"
#include "commands.h"
#include "V_DEF_MCP19111.h"
#include "ctype.h"
#include "string.h"
#include "stdlib.h"
#include "macro.h"
#include "flash_rw.h"
#include "pmbus_core.h"
#include "pmbus_handler.h"
#include "adc_monitor.h"
#include "init_part.h"
#include "output_mngt.h"

__CONFIG(WDTE_OFF & PWRTE_ON & MCLRE_ON & CP_OFF & WRT_OFF);

//variables
bit m_VINStopOutput = 0;
bit m_OVStopOutput = 0;
bit m_UVStopOutput = 0;
bit m_OCStopOutput = 0;

unsigned int restoreUVFilter = 0;
unsigned int restoreOCFilter = 0;
unsigned int restoreOVFilter = 0;
unsigned int restoreEnPin = DEF_EN_PIN_RECOVERY_TIME;

unsigned char delayUVFilter = 0;
unsigned char delayOVFilter = 0;
unsigned char delayOCFilter = 0;
unsigned char m_mainLoopReq = DEF_MAIN_LOOP_REQ_INIT;
unsigned char respTriedNumber=0;
unsigned char uvOvResp=0;

int intTmp = 0;

void main() {

    //Init main peripherals
    initPart();
    //Init warning threshold value
    initADCVal();
    
#if V_PMB_INIT == 1 //PMBUS init
    initPMBusStack();
    applyAllPMBSettings(V_PMB_FLASH_INIT);
#else
    if (((OPERATION[0]) & OPERATION_ENABLE) == OPERATION_ENABLE)
        restoreOutput();
#endif

    for (;;) {

#if V_PMB_INIT == 1
        //----------------------------------------------------------
        //PMBus part: update part after a write command if needed
        //----------------------------------------------------------
        // buffer ready to be copied?
        if (i2c_flags.ready_to_copy) {
            i2c_flags.ready_to_copy = 0; // clear flag
            pmbusHostWrite(); // call copy function
        }
        if (i2c_flags.apply_all_req) {
            i2c_flags.apply_all_req = 0; // clear flag
            applyAllPMBSettings(0); // call copy function
        }
        //run I2C watchdog
        PMBusCheckTimeout();

        //set the SMBAlert pin high if needed
        if (STATUS_WORD[1] | (STATUS_WORD[0] & ~STATUS_OFF))
        {
            SSPADD2 = V_SLAVE_ALERT_ADDR;
            SMBALERT_PIN_IO = 0;
        }
        else
        {
            SSPADD2 = SLAVE_ALERT_DIS_ADDR;
            SMBALERT_PIN_IO = 1;
        }
#endif
        //----------------------------------------------------------
        //Enable the timer for output ramp if the output doesn't match the target
        //----------------------------------------------------------
        if (m_OVCCON_target != OVCCON) //Restore output is need
            TMR1ON = 1; //enabled timer

        //----------------------------------------------------------
        //Recover from protection (Over voltage) part
        //----------------------------------------------------------
        if (m_OVStopOutput) //Output stopped because OV?
        {
            if (!OVIF) //OV no more present?
            {
                //Restart delay ended?
                if (restoreOVFilter > DEF_OV_UV_RECOVERY_TIME) {
                    m_OVStopOutput = 0;
                    //Check if restart mode is enabled
                    respTriedNumber = GET_RESP_TRIED(VOUT_OV_FAULT_RESPONSE[0]);
                    if (respTriedNumber > 0)
                    {
                        restoreOutput();
                        //If restart number is not infinit, sub one tried
                        if (respTriedNumber < RESP_TRIED_MASK)
                            SUB_RESP_TRIED(VOUT_OV_FAULT_RESPONSE[0]);
                    }
                    restoreOVFilter = 0;
                } else
                    restoreOVFilter++;
            }
        }

        //----------------------------------------------------------
        //Recover from protection (Under voltage) part
        //----------------------------------------------------------
        if (m_UVStopOutput) //Output stopped because UV?
        {
            if (!UVIF) //UV no more present?
            {
                //Restart delay ended?
                if (restoreUVFilter > DEF_OV_UV_RECOVERY_TIME) {
                    m_UVStopOutput = 0;
                    //Check if restart mode is enabled
                    respTriedNumber = GET_RESP_TRIED(VOUT_UV_FAULT_RESPONSE[0]);
                    if (respTriedNumber > 0)
                    {
                        restoreOutput();
                        //If restart number is not infinit, sub one tried
                        if (respTriedNumber < RESP_TRIED_MASK)
                            SUB_RESP_TRIED(VOUT_UV_FAULT_RESPONSE[0]);
                    }
                    restoreUVFilter = 0;
                } else
                    restoreUVFilter++;
            }
        }

        //----------------------------------------------------------
        //Recover from protection (Vin lockout) part
        //----------------------------------------------------------
        if (m_VINStopOutput) //Output stopped because VIN UV?
        {
            if (!VINIF) //VINUV no more present?
            {
                m_VINStopOutput = 0;
                intTmp = GET_MANTISSA(VIN_OFF)/VIN_TABLE_INPUT_DIV;
                VINLVL = (vinLookupTable[intTmp]) | VIN_UV_EN;
                //restoreOutput(); uncomment this line to enable auto restore
            }
        }

        //----------------------------------------------------------
        //Recover from protection (Over current) part
        //----------------------------------------------------------
        if (m_OCStopOutput) //Output stopped because OC?
        {
            if (!OCIF) //OC no more present?
            {
                //Restart delay ended?
                if (restoreOCFilter > DEF_OV_UV_RECOVERY_TIME) {
                    m_OCStopOutput = 0;
                    //Check if restart mode is enabled
                    respTriedNumber = GET_RESP_TRIED(IOUT_OC_FAULT_RESPONSE[0]);
                    if (respTriedNumber > 0)
                    {
                        restoreOutput();
                        //If restart number is not infinit, sub one tried
                        if (respTriedNumber < RESP_TRIED_MASK)
                            SUB_RESP_TRIED(IOUT_OC_FAULT_RESPONSE[0]);
                    }
                    restoreOCFilter = 0;
                } else
                    restoreOCFilter++;
            }
        }

        //----------------------------------------------------------
        //Check values from ADC
        //----------------------------------------------------------
        runAdcMonitor();

        //----------------------------------------------------------
        //Re-activate peripharals protections interruption if needed
        //----------------------------------------------------------
        if (m_mainLoopReq)
            m_mainLoopReq--;
        else
            PIE2 = DEF_PIE2;

        if (restoreEnPin)
            restoreEnPin--;

    }
}

/********************************************************************
 * Function:        	ISR
 *
 * Overview:        	MCU's interrupt service routine.
 *******************************************************************/
void interrupt isr(void) {

    PIE2 = 0x00; 
    //Remove peripherals interrupt until main loop re-enable it
    //because we do not want the isr to block the main loop.
    m_mainLoopReq = 1;

    //----------------------------------------------------------
    if (SSPIF) { //SERIAL COMMUNICATION
#if V_PMB_INIT == 1
        i2cSlaveHandler();
#endif
    }
    //----------------------------------------------------------
    if (IOCF) { //IO CHANGE
        if (!restoreEnPin)
            checkEnPin();
        IOCF = 0;
    }
    //----------------------------------------------------------
    if (TMR1IF) { //TIMER 1 COMPLET
        TMR1IF = 0;
        voutRampTarget();
    }
    //----------------------------------------------------------
    if (BCLIF) { //SERIAL COMMUNICATION COLLISION
#if V_PMB_INIT == 1
        STATUS_WORD[0] |= COM_OTHER_COM_FAULT; //Invalid or unsupported data OR
        STATUS_CML[0] |= COM_DATA_FAULT; //Data out of range
#endif
        BCLIF = 0;
    }
    //----------------------------------------------------------
    if (VINIF) { //VIN LOCKOUT
        //Flag to advise the main loop we will stop the output because of the VIN UV
        intTmp = GET_MANTISSA(VIN_ON)/VIN_TABLE_INPUT_DIV;
        VINLVL = (vinLookupTable[intTmp]) | 0x80;
        softStopOutput();
        m_VINStopOutput = 1;
        if (m_OVCCON_target == OVCCON)
        {
            STATUS_WORD[0] |= STATUS_VIN_FAULT;
            STATUS_WORD[1] |= STATUS_VIN_ERR;
        }
        VINIF = 0;
    }
    //----------------------------------------------------------
    //OUTPUT UNDER VOLTAGE
    unsigned char ignoreFaultCond = ((*OPERATION) & OPERATION_IGNORE_FAULT && (*OPERATION) & OPERATION_MARGIN_MASK);
    if (UVIF &&  !ignoreFaultCond) {
        uvOvResp = GET_RESP_MODE(VOUT_UV_FAULT_RESPONSE[0]);
        switch (uvOvResp)
        {
            case RESTART_IF_FAULT_AGAIN:
                if (delayUVFilter == DEF_OV_UV_DELAY_TIME)
                {
                    m_UVStopOutput = 1;
                    softStopOutput();
                } else
                    delayUVFilter++;
                break;
            case DIRECT_RESTART_IF_FAULT:
                m_UVStopOutput = 1;
                softStopOutput();
                break;
            case DIRECT_STOP:
                softStopOutput();
                break;
        }
#if V_PMB_INIT == 1
        STATUS_VOUT[0] |= OV_UV_FAULT;
        STATUS_WORD[1] |= STATUS_OV_ERR;
#endif
        UVIF = 0;
    } else
        delayUVFilter=0;
    //----------------------------------------------------------
    //OUTPUT OVER VOLTAGE
    if (OVIF && !ignoreFaultCond) {
        uvOvResp = GET_RESP_MODE(VOUT_OV_FAULT_RESPONSE[0]);
        switch (uvOvResp)
        {
            case RESTART_IF_FAULT_AGAIN:
                if (delayOVFilter == DEF_OV_UV_DELAY_TIME)
                {
                    m_OVStopOutput = 1;
                    softStopOutput();
                } else
                  delayOVFilter++;
                break;
            case DIRECT_RESTART_IF_FAULT:
                m_OVStopOutput = 1;
                softStopOutput();
                break;
            case DIRECT_STOP:
                softStopOutput();
                break;
        }
#if V_PMB_INIT == 1
        STATUS_WORD[0] |= STATUS_OV_OV_FAULT;
        STATUS_WORD[1] |= STATUS_OV_ERR;
        STATUS_VOUT[0] |= OV_OV_FAULT;
#endif
        OVIF = 0;
    }else
        delayOVFilter=0;
    //----------------------------------------------------------
    //OUTPUT OVER CURRENT
    if (OCIF && !ignoreFaultCond) {
        uvOvResp = GET_RESP_MODE(IOUT_OC_FAULT_RESPONSE[0]);
        switch (uvOvResp)
        {
            case RESTART_IF_FAULT_AGAIN:
                if (delayOCFilter == DEF_OV_UV_DELAY_TIME)
                {
                    m_OCStopOutput = 1;
                    softStopOutput();
                } else
                  delayOCFilter++;
                break;
            case DIRECT_RESTART_IF_FAULT:
                m_OCStopOutput = 1;
                softStopOutput();
                break;
            case DIRECT_STOP:
                softStopOutput();
                break;
        }
        STATUS_WORD[0] |= STATUS_OC_FAULT;
        STATUS_WORD[1] |= STATUS_OP_ERR;
        OCIF = 0;
    }else
        delayOCFilter=0;
}



