 /********************************************************************
 * FileName:        output_mngt.c
 * Dependencies:    See INCLUDES section below
 * Processor:       MCP19111
 * Compiler:        XC8 v1.11
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement:
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") for its PICmicro Microcontroller is intended and
 * supplied to you, the Company's customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in vsiolation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *********************************************************************
 *
 * Version              Date        Comment
 * 1.000                05.12.2012  Initial release
 ********************************************************************/

#include "output_mngt.h"


#if V_PMB_INIT == 1
int intTmpCheck=0;
#endif

unsigned char m_TMR1H = V_TMR1H;
unsigned char m_TMR1L = V_TMR1L;
unsigned char rampCpt=0;



/********************************************************************
 * Function:        	OVRamp
 *
 * Overview:        	First, this function stop the timer 1.
 *                      If the soft start is NOT enabled, the funtion directy
 *                      set the ouput voltage and protection.
 *                      If the soft start is enabled, the funtion will increase
 *                      the output voltage level and set the correct timer 1
 *                      value for next interrupt. It will also re-enable
 *                      the timer1. This function is called at each timer 1
 *                      interrupt.
 *
 *                      When the output voltage level reaches its
 *                      desired level, timer 1 is not re-enabled.
 *                      It also enable output protections at the end of the ramp
 *******************************************************************/
void voutRampTarget(void) {
    TMR1ON = 0; //disabled timer

    if (OVCCON != m_OVCCON_target)
    {
        ABECON = V_ABECON & NOT_OV_UV_MASK;
        m_softWarningCheckEn=0;
        TMR1H = m_TMR1H;
        TMR1L = m_TMR1L;
        TMR1ON = 1; //enabled timer
    }
    else if (!DRVDIS) //not enable on
    {
        ABECON = V_ABECON; //restore output protection
        m_softWarningCheckEn=1;
    }

    for (rampCpt = RAMP_STEP; rampCpt > 0; rampCpt--)
    {
        if (OVCCON > m_OVCCON_target)
            OVCCON--;
        if (OVCCON < m_OVCCON_target)
            OVCCON++;
    }
    if (OVCCON > 0)
    {
        DRVDIS = V_ATSTCON & 0x01;
        STATUS_WORD[0] &= ~STATUS_OFF;
    }
    else
    {
        DRVDIS = 1; //Disabled driver when output = 0V
        STATUS_WORD[0] |= STATUS_OFF;
    }
}

/********************************************************************
 * Function:        	switchOffOutput
 *
 * Overview:        	Stop switching without soft stop and disable output
 *                      protection
 *******************************************************************/
void switchOffOutput(void) {
    softStopOutput();
    ABECON = V_ABECON & NOT_OV_UV_MASK;
    DRVDIS = 1;
    OVCCON = 0;
}

/********************************************************************
 * Function:        	restoreOutput
 *
 * Overview:        	Set the output voltage:
 *                      -With soft start if enable
 *                      -Set output voltage from the GUI at first use
 *                      -Restore last output voltage if previously stopped
 *******************************************************************/
void restoreOutput(void) {

#if V_PMB_INIT == 1
    if ( ((*OPERATION) & OPERATION_NORMAL) == 0 )
        intTmpCheck = (GET_MANTISSA(VOUT_COMMAND));
    else if ((*OPERATION) & OPERATION_MARGIN_LOW)
        intTmpCheck = (GET_MANTISSA(VOUT_MARGIN_LOW));
    else if ((*OPERATION) & OPERATION_MARGIN_HIGH)
        intTmpCheck = (GET_MANTISSA(VOUT_MARGIN_HIGH));

    m_OVCCON_target = VOUT_CODE_FROM_PMB(intTmpCheck);
#else
    m_OVCCON_target = V_OVCCON;
#endif
    OPERATION[0] |= OPERATION_ENABLE;
    TMR1ON = 1;
}

/********************************************************************
 * Function:        	softStopOutput
 *
 * Overview:        	Disabled the output voltage with soft-stop:
 *                      -Store the current output voltage in case of
 *                       future restoration.
 *                      -Disable the output protection
 *                      -Set the new new output target to 0 and enabled Timer1
 *******************************************************************/
void softStopOutput(void) {
    OVFCON &= 0x80;
    m_OVCCON_target = 0;
    TMR1ON = 1;
    OPERATION[0] &= ~OPERATION_ENABLE;
}

/********************************************************************
 * Function:        	checkOutputRange
 *
 * PreCondition:        This function assume that the VOUT PMBus exponent
 *                      is fixed to -10 and PMBus is enabled.
 *
 * Overview:        	Convert the VOUT PMBus value to code value
 *                      Check if the code value is inside the allowed range
 *                      If so, place the code value in the approritate register
 *******************************************************************/
#if V_PMB_INIT == 1
void checkOutputRange(unsigned char * pmb_mem, unsigned char * reg) {
    intTmpCheck = VOUT_GET_MANTISSA(pmb_mem);
    //Verify if the value is inside the authorised range
    if (VOUT_CODE_FROM_PMB_OVERFLOW(intTmpCheck))
        transContentFaultHandler(3);
    else
        *reg = VOUT_CODE_FROM_PMB(intTmpCheck);

}
#endif
/********************************************************************
 * Function:        	checkEnPin
 *
 * Overview:        	Enable or not the output in function of the pin
 *                      stat and the device configuration.
 *******************************************************************/

void checkEnPin(void) {
    if (((*ON_OFF_CONFIG) & ON_OFF_PIN_ON) && ((*ON_OFF_CONFIG) & ON_OFF_ALWAYS_ON_B))
    {
        if (EN_PIN == (((*ON_OFF_CONFIG) & ON_OFF_PIN_POLARITY)) >> 1)
        {
            if (OPERATION[0] & OPERATION_ENABLE)
                softStopOutput();
            else
                restoreOutput();
        }
    }
}