 /********************************************************************
 * FileName:        pmbus_handler.c
 * Dependencies:    See INCLUDES section below
 * Processor:       MCP19111
 * Compiler:        XC8 v1.11
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement:
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") for its PICmicro Microcontroller is intended and
 * supplied to you, the Company's customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in vsiolation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *********************************************************************
 *
 * Version              Date        Comment
 * 1.000                05.12.2012  Initial release
 ********************************************************************/



#include "pmbus_handler.h"

#if V_PMB_INIT == 1

int intTmp1=0;
unsigned int uintTmp2=0;

/********************************************************************
 * Function:        	pmbusHostRead
 *
 * PreCondition: 	None
 *
 * Input:        	The parameters are the code of the COMMAND under process
 *			and the pointer to the location in RAM of the COMMAND
 *			data(pointer to the small array associated with the
 *			COMMAND data. EX : MFR_VIN_MIN[2])
 *
 * Overview:        	This function is provided as an example. This function is
 *			defined in the application and called by the stack as
 *			a BlockWriteBlockReadProcessCall PMBus transaction is on course
 *			and the application is required to process the bytes received
 *			so far by the slave and prepare data to be sent back to the
 *			host. The prepared data has to be stored in the COMMAND's RAM
 *			for the stack to retrieve it from there.
 * Note:        	None
 *******************************************************************/

void pmbusHostRead(unsigned char code, unsigned char *ram_index) {
    //Prepare data from SFR before sending if needed
    int bufInt=0;

    switch (code) {
        case READ_VIN_COMMAND:
            bufInt = m_adcValueList[ADC_VAL_VIN_ANA][0]/4; //Read value
            bufInt = bufInt + bufInt/16;
            READ_VIN[0] = LOW_BYTE(bufInt);
            READ_VIN[1] = (HIGH_BYTE(bufInt) & 0x7) | VIN_FIXED_EXP;
            break;
        case READ_VOUT_COMMAND:
            bufInt = m_adcValueList[ADC_VAL_INT_VREG][0]*5/4; //Read value
            READ_VOUT[0] = LOW_BYTE(bufInt);
            READ_VOUT[1] = HIGH_BYTE(bufInt);
            break;
        case READ_TEMPERATURE_1_COMMAND:
            bufInt = m_adcValueList[ADC_VAL_TMP_ANA][0]; //Read value
            bufInt = (bufInt*2/5) - 142;
            READ_TEMPERATURE_1[0] = LOW_BYTE(bufInt);
            READ_TEMPERATURE_1[1] = (HIGH_BYTE(bufInt) & 0x7) | TEMP_FIXED_EXP;
            break;
        default:
            break;
    }
}

/********************************************************************
 * Function:		writeAllInFlash
 *
 * PreCondition: 	PMBus stack enabled
 *
 * Overview:		This function will write the PMBus stack
 *                    RAM content into the Flash memory.
 *******************************************************************/
void writeAllInFlash()
{
    int i, j;
    int writeCpt=0;
    int writeIndex=0;
    unsigned int writeBuff[WRITE_SIZE];
    //Check all command...
    for (i=0; i < NR_COMMANDS; i++) {
        if (matrix[i].inFlash != NO_FLASH) //...If they have a Flash address
        {
            for (j=0; j < matrix[i].nrBytesWr; j++)//Filling write fly buffer
            {
                writeBuff[writeCpt++]= *((matrix[i].ptrCommandData) + j);
                if (writeCpt==WRITE_SIZE) //If buffer is full
                {
                    writeFlash(writeIndex, writeBuff);
                    writeIndex+=4;
                    writeCpt=0;
                }
            }
        }
    }
    //Finish last group if needed
    if (writeCpt)
        writeFlash(writeIndex, writeBuff);
}

/********************************************************************
 * Function:		restoreAllFromFlash
 *
 * PreCondition: 	PMBus values were previously stored in Flash
 *
 * Overview:		This function will replace the PMBus stack
 *                    RAM content by values that were previously
 *                    stored in Flash memory.
 *******************************************************************/
void restoreAllFromFlash()
{
    int i, j;
    char bufTmp[2];
    for (i=0; i < NR_COMMANDS; i++) {
        if (matrix[i].inFlash != NO_FLASH)
        {
            for (j=0; j < matrix[i].nrBytesWr; j++)
            {
                readFlash(matrix[i].inFlash+j, bufTmp, 0);
                *((matrix[i].ptrCommandData) + j) = bufTmp[0];
            }
        }
    }
}


/********************************************************************
 * Function:		pmbusHostWrite
 *
 * PreCondition: 	A successful PMBus write transaction took place.
 *
 * Overview:		Following a PMBUS COMMAND being received in a write
 *			PMBus transaction, this function copies the content
 *			of the temp BUFFER in the appropriate RAM location
 *			of the COMMAND
 *******************************************************************/

void pmbusHostWrite() {
    //Write data in SFR if host write and if need
        switch (m_codeCMD) {
            case OPERATION_COMMAND:
                if (((*ON_OFF_CONFIG) & ON_OFF_I2C_ON) && ((*ON_OFF_CONFIG) & ON_OFF_ALWAYS_ON_B))
                {
                    if ((OPERATION[0] & OPERATION_ENABLE) == OPERATION_ENABLE)
                        restoreOutput();
                    else
                        softStopOutput();
                }
                break;
            case ON_OFF_CONFIG_COMMAND:
                if (((*ON_OFF_CONFIG) & ON_OFF_ALWAYS_ON_B) == 0)
                {
                    if ((OPERATION[0] & OPERATION_ENABLE) == OPERATION_ENABLE)
                        restoreOutput();
                }
                break;
            case MAX_DUTY_COMMAND:
            case INTERLEAVE_COMMAND:
            case FREQUENCY_SWITCH_COMMAND:
                intTmp1 = GET_MANTISSA(FREQUENCY_SWITCH);
                intTmp1=(4000/intTmp1);
                PR2 = intTmp1;
                intTmp1 = (INTERLEAVE[0]);
                unsigned char sum = (intTmp1 & 0xF0)>>4;
                unsigned char num = (intTmp1 & 0xF);
                PWMPHL = (num*PR2/sum)%(PR2+1);
                intTmp1 = (MAX_DUTY[0]);
                PWMRL = (intTmp1*PR2/100+PWMPHL)%(PR2+1);
                break;

            case CLEAR_FAULTS_COMMAND:
                clearFaults();
                break;
            case STORE_DEFAULT_ALL_COMMAND:
                writeAllInFlash();
                break;
            case RESTORE_DEFAULT_ALL_COMMAND:
                restoreAllFromFlash();
                i2c_flags.apply_all_req=1;
                break;
            case VIN_ON_COMMAND:
                if (VINIF)
                {
                    intTmp1 = GET_MANTISSA(VIN_ON)/VIN_TABLE_INPUT_DIV;
                    //Verify if the value is inside the authorised range
                    if (intTmp1>=VIN_TABLE_SIZE)
                        transContentFaultHandler(3);
                    else
                    {
                        intTmp1 = vinLookupTable[intTmp1];
                        VINLVL = intTmp1 | 0x80;
                    }
                }
                break;
            case VIN_OFF_COMMAND:
                if (VINIF==0)
                {
                    intTmp1 = GET_MANTISSA(VIN_OFF)/VIN_TABLE_INPUT_DIV;
                    //Verify if the value is inside the authorised range
                    if (intTmp1>=VIN_TABLE_SIZE)
                        transContentFaultHandler(3);
                    else
                    {
                        intTmp1 = vinLookupTable[intTmp1];
                        VINLVL = intTmp1 | 0x80;
                    }
                }
                break;
            case VOUT_COMMAND_COMMAND:
            {
                if ((*OPERATION) & OPERATION_ENABLE)
                    checkOutputRange(VOUT_COMMAND, &m_OVCCON_target);
                break;
            }
            case VOUT_MARGIN_HIGH_COMMAND:
            {
                checkOutputRange(VOUT_MARGIN_HIGH, DUMMY);
                break;
            }
            case VOUT_MARGIN_LOW_COMMAND:
            {
                checkOutputRange(VOUT_MARGIN_LOW, DUMMY);
                break;
            }
            case VOUT_OV_FAULT_LIMIT_COMMAND:
            {
                checkOutputRange(VOUT_OV_FAULT_LIMIT, (unsigned char *)&OOVCON);
                break;
            }
            case VOUT_UV_FAULT_LIMIT_COMMAND:
            {
                checkOutputRange(VOUT_UV_FAULT_LIMIT, (unsigned char *)&OUVCON);
                break;
            }
            case IOUT_OC_FAULT_LIMIT_COMMAND:
            {
                intTmp1 = GET_MANTISSA(IOUT_OC_FAULT_LIMIT);
                intTmp1 = ((intTmp1*V_SETTING_RON)-160)/15;
                if (intTmp1<0)
                {
                    intTmp1=0;
                    transContentFaultHandler(3);
                }
                //Verify if the value is inside the authorised range
                if (intTmp1>=64)
                {
                    intTmp1=63;
                    transContentFaultHandler(3);
                }
                else
                    OCCON = intTmp1;
                break;
            }
            case POWER_GOOD_ON_COMMAND:
            case POWER_GOOD_OFF_COMMAND:
            case OT_WARN_LIMIT_COMMAND:
                initADCVal();
                break;
            case TON_RISE_COMMAND:
            {
                STATUSbits.CARRY=0;
                STATUSbits.ZERO=0;
                intTmp1 = GET_MANTISSA(TON_RISE)*4; //(ms)
                uintTmp2 = GET_MANTISSA(VOUT_COMMAND)/64;
                if (STATUSbits.CARRY || uintTmp2==0)
                {
                    transContentFaultHandler(3);
                }
                else
                {
                    uintTmp2 = intTmp1/uintTmp2;
                    uintTmp2=~uintTmp2;
                    m_TMR1L = LOW_BYTE(uintTmp2);
                    m_TMR1H = HIGH_BYTE(uintTmp2);
                }
                break;
            }
    }
}

#endif
